import React from 'react';
import type { ReactionResponse } from 'stream-chat';
import type { ReactEventHandler } from '../Message/types';
import type { DefaultStreamChatGenerics } from '../../types/types';
import type { ReactionOptions } from './reactionOptions';
import type { ReactionDetailsComparator, ReactionsComparator } from './types';
import { MessageContextValue } from '../../context';
export type ReactionsListProps<StreamChatGenerics extends DefaultStreamChatGenerics = DefaultStreamChatGenerics> = Partial<Pick<MessageContextValue<StreamChatGenerics>, 'handleFetchReactions'>> & {
    /** Custom on click handler for an individual reaction, defaults to `onReactionListClick` from the `MessageContext` */
    onClick?: ReactEventHandler;
    /** An array of the own reaction objects to distinguish own reactions visually */
    own_reactions?: ReactionResponse<StreamChatGenerics>[];
    /** An object that keeps track of the count of each type of reaction on a message */
    reaction_counts?: Record<string, number>;
    /** A list of the currently supported reactions on a message */
    reactionOptions?: ReactionOptions;
    /** An array of the reaction objects to display in the list */
    reactions?: ReactionResponse<StreamChatGenerics>[];
    /** Display the reactions in the list in reverse order, defaults to false */
    reverse?: boolean;
    /** Comparator function to sort the list of reacted users, defaults to alphabetical order */
    sortReactionDetails?: ReactionDetailsComparator;
    /** Comparator function to sort reactions, defaults to alphabetical order */
    sortReactions?: ReactionsComparator;
};
/**
 * Component that displays a list of reactions on a message.
 */
export declare const ReactionsList: <StreamChatGenerics extends DefaultStreamChatGenerics = DefaultStreamChatGenerics>(props: ReactionsListProps<StreamChatGenerics>) => React.JSX.Element | null;
//# sourceMappingURL=ReactionsList.d.ts.map