import { __assign, __rest } from "tslib";
import React, { useMemo } from 'react';
import clsx from 'clsx';
import { Modal } from '../Modal';
import { useFetchReactions } from './hooks/useFetchReactions';
import { LoadingIndicator } from '../Loading';
import { Avatar } from '../Avatar';
import { useMessageContext } from '../../context';
var defaultSortReactionDetails = function (a, b) {
    var _a, _b, _c, _d, _e, _f;
    var aName = (_b = (_a = a.user) === null || _a === void 0 ? void 0 : _a.name) !== null && _b !== void 0 ? _b : (_c = a.user) === null || _c === void 0 ? void 0 : _c.id;
    var bName = (_e = (_d = b.user) === null || _d === void 0 ? void 0 : _d.name) !== null && _e !== void 0 ? _e : (_f = b.user) === null || _f === void 0 ? void 0 : _f.id;
    return aName ? (bName ? aName.localeCompare(bName, 'en') : -1) : 1;
};
export function ReactionsListModal(_a) {
    var _b, _c;
    var handleFetchReactions = _a.handleFetchReactions, onSelectedReactionTypeChange = _a.onSelectedReactionTypeChange, reactions = _a.reactions, selectedReactionType = _a.selectedReactionType, propSortReactionDetails = _a.sortReactionDetails, modalProps = __rest(_a, ["handleFetchReactions", "onSelectedReactionTypeChange", "reactions", "selectedReactionType", "sortReactionDetails"]);
    var selectedReaction = reactions.find(function (_a) {
        var reactionType = _a.reactionType;
        return reactionType === selectedReactionType;
    });
    var SelectedEmojiComponent = (_b = selectedReaction === null || selectedReaction === void 0 ? void 0 : selectedReaction.EmojiComponent) !== null && _b !== void 0 ? _b : null;
    var _d = useFetchReactions({
        handleFetchReactions: handleFetchReactions,
        shouldFetch: modalProps.open,
    }), areReactionsLoading = _d.isLoading, allReactions = _d.reactions;
    var contextSortReactionDetails = useMessageContext('ReactionsListModal').sortReactionDetails;
    var sortReactionDetails = (_c = propSortReactionDetails !== null && propSortReactionDetails !== void 0 ? propSortReactionDetails : contextSortReactionDetails) !== null && _c !== void 0 ? _c : defaultSortReactionDetails;
    var currentReactions = useMemo(function () {
        if (!selectedReactionType) {
            return [];
        }
        var unsortedCurrentReactions = allReactions.filter(function (reaction) { return reaction.type === selectedReactionType && reaction.user; });
        return unsortedCurrentReactions.sort(sortReactionDetails);
    }, [allReactions, selectedReactionType, sortReactionDetails]);
    return (React.createElement(Modal, __assign({}, modalProps),
        React.createElement("div", { className: 'str-chat__message-reactions-details', "data-testid": 'reactions-list-modal' },
            React.createElement("div", { className: 'str-chat__message-reactions-details-reaction-types' }, reactions.map(function (_a) {
                var EmojiComponent = _a.EmojiComponent, reactionCount = _a.reactionCount, reactionType = _a.reactionType;
                return EmojiComponent && (React.createElement("div", { className: clsx('str-chat__message-reactions-details-reaction-type', {
                        'str-chat__message-reactions-details-reaction-type--selected': selectedReactionType === reactionType,
                    }), "data-testid": "reaction-details-selector-".concat(reactionType), key: reactionType, onClick: function () { return onSelectedReactionTypeChange === null || onSelectedReactionTypeChange === void 0 ? void 0 : onSelectedReactionTypeChange(reactionType); } },
                    React.createElement("span", { className: 'emoji str-chat__message-reaction-emoji str-chat__message-reaction-emoji--with-fallback' },
                        React.createElement(EmojiComponent, null)),
                    "\u00A0",
                    React.createElement("span", { className: 'str-chat__message-reaction-count' }, reactionCount)));
            })),
            SelectedEmojiComponent && (React.createElement("div", { className: 'emoji str-chat__message-reaction-emoji str-chat__message-reaction-emoji--with-fallback str-chat__message-reaction-emoji-big' },
                React.createElement(SelectedEmojiComponent, null))),
            React.createElement("div", { className: 'str-chat__message-reactions-details-reacting-users', "data-testid": 'all-reacting-users' }, areReactionsLoading ? (React.createElement(LoadingIndicator, null)) : (currentReactions.map(function (_a) {
                var user = _a.user;
                return (React.createElement("div", { className: 'str-chat__message-reactions-details-reacting-user', key: user === null || user === void 0 ? void 0 : user.id },
                    React.createElement(Avatar, { "data-testid": 'avatar', image: user === null || user === void 0 ? void 0 : user.image, name: (user === null || user === void 0 ? void 0 : user.name) || (user === null || user === void 0 ? void 0 : user.id) }),
                    React.createElement("span", { className: 'str-chat__user-item--name', "data-testid": 'reaction-user-username' }, (user === null || user === void 0 ? void 0 : user.name) || (user === null || user === void 0 ? void 0 : user.id))));
            }))))));
}
